<?php
/**
 * SynergeticAgency_Gls
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magento.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Magento to newer
 * versions in the future. If you wish to customize Magento for your
 * needs please refer to http://www.magento.com for more information.
 *
 * @category   SynergetigAgency
 * @package    SynergeticAgency\Gls\Model\Shipment
 * @copyright  Copyright (c) 2016 synergetic agency AG (http://agency.synergetic.ag)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

/**
 * GLS shipment returnlabel model
 * this model is just a variation of the shipment model
 *
 * @category    SynergeticAgency
 * @package     SynergeticAgency_Gls
 * @author      PHP WebDevelopment <php.webdevelopment@synergetic.ag>
 */
class SynergeticAgency_Gls_Model_Shipment_Returnlabel extends SynergeticAgency_Gls_Model_Shipment
{

    /**
     * init
     */
    protected function _construct() 
    {
        // we use the shipment resourc because this model is a variation of the shipment model
        $this->_init('synergeticagency_gls/shipment');
    }

    /**
     * // we don't want saving in this object at the moment
     * // use the original shipment model for that
     * @return $this
     */
    public function save() 
    {
        return $this;
    }

    /**
     * @return array
     */
    public function getLabels() 
    {
        $connector = $this->getConnector();
        $connectorShipment = $this->createConnectorShipment();
        $this->setConnectorShipment($connectorShipment);

        // set return address as delivery address because the delivery address is
        $returnDeliveryAddress = $this->createAddressFromReturnAddress(SynergeticAgency_GlsConnector_Model_Address::GLS_API_ADDRESS_DELIVERY);
        $connectorShipment->addAddress($returnDeliveryAddress);
        // set delivery address as alternative shipper address because the shipper is the client in this case
        $returnShipperAddress = $this->createAddressFromDeliveryAddress(SynergeticAgency_GlsConnector_Model_Address::GLS_API_ADDRESS_ALTERNATIVESHIPPING);
        $connectorShipment->addAddress($returnShipperAddress);

        $connectorParcels = $this->createConnectorParcels(true);
        foreach($connectorParcels as $connectorParcel) {
            $connectorShipment->pushParcel($connectorParcel);
        }

        $connector->setShipment($connectorShipment);
        Mage::helper("synergeticagency_gls/log")->logApiCommunication(__METHOD__, __LINE__, $connector->getShipment()->toJson());

        try {
            // send shipment to the GLS-API
            $connector->createShipment();
            $shipmentResponse = $connector->getShipment();
            $returnLabels = $shipmentResponse->getLabels();
        } catch(Exception $e) {
            // handle exceptions of the connector
            Mage::helper("synergeticagency_gls/log")->log(__METHOD__, __LINE__, $e->getMessage().' '.$e->getCode(), Zend_Log::ERR);
            // if something breaks don't stop operation just log and return empty return label
            $returnLabels = array();
        }

        Mage::helper("synergeticagency_gls/log")->logApiCommunication(__METHOD__, __LINE__, $connector->getCurlResponse());
        return $returnLabels;
    }


    /**
     * @return array
     */
    protected function createConnectorParcels() 
    {
        $createdParcels = array();

        $service = new SynergeticAgency_GlsConnector_Model_Service();
        $service->setName(SynergeticAgency_GlsConnector_Model_Service::GLS_API_SERVICE_SHOPRETURNSERVICE);

        $info = new SynergeticAgency_GlsConnector_Model_Info();
        $info->setName('returnonly');
        $info->setValue('Y');
        $service->pushInfo($info);

        foreach($this->getShipmentParcels() as $parcel) {
            $connectorParcel = new SynergeticAgency_GlsConnector_Model_Parcel();
            $connectorParcel->setWeight($parcel->getWeight());
            $connectorParcel->setReferences(array($parcel->getId()));
            $connectorParcel->pushService($service);

            $createdParcels[] = $connectorParcel;
        }

        return $createdParcels;
    }
}